package com.gjz.xiaoheizi.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gjz.xiaoheizi.controller.dto.FilesDto;
import com.gjz.xiaoheizi.controller.dto.MerchantDto;
import com.gjz.xiaoheizi.entity.Callout;
import com.gjz.xiaoheizi.entity.Merchant;
import com.gjz.xiaoheizi.entity.User;
import com.gjz.xiaoheizi.mapper.FilesMapper;
import com.gjz.xiaoheizi.mapper.MerchantMapper;
import com.gjz.xiaoheizi.service.CalloutService;
import com.gjz.xiaoheizi.service.MerchantService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Guan
 * @date 2022/10/19
 * discription
 */
@RestController
@RequestMapping("/merchant")
public class MerchantController {
    @Autowired
    private MerchantService merchantService;

    @Autowired
    private MerchantMapper merchantMapper;

    @Autowired
    private FilesMapper filesMapper;

    @Autowired
    private CalloutService calloutService;
    @Value("${files.upload.path}")
    private String fileUploadPath;

    @GetMapping("/{ids}")
    public MerchantDto getsss(@PathVariable Integer ids) {
//        List<MerchantDto> byId = merchantMapper.findById(ids);
        MerchantDto byId = merchantMapper.findById(ids);
        return byId;
    }
//    @GetMapping("/merchantAndCallout")
//    public List<MerchantDto> getAll(){
//        List<Merchant> idList = merchantService.list();
//        List<MerchantDto> merchantDtos = new ArrayList<>();
//        for (int i = 0; i < idList.size(); i++) {
//            Integer id = idList.get(i).getId();
//            MerchantDto byId = merchantMapper.findById(id);
//            merchantDtos.add(byId);
//        }
//        return merchantDtos;
//    }

    //查询所有商家信息
    @GetMapping
    public List<Merchant> findAll(){
        return merchantService.list();
    }

    //修改标志位
    @GetMapping("/status/{id}")
    public void statusChange(@PathVariable Integer id){
        Merchant byId = merchantService.getById(id);
        int status = byId.getStatus();
        if (status == 0){
            status = 1;
        }else {
            status = 0;
        }
        byId.setStatus(status);
        merchantService.updateById(byId);
    }

    //新增和修改
    @PostMapping
    public boolean save(@RequestBody Merchant merchant){
        if (merchantService.getById(merchant.getId()) != null){
            Callout callout = new Callout();
            callout.setContent(merchant.getTitle());
            callout.setMerchantId(merchant.getId());
            calloutService.updateContent(callout);
            return merchantService.saveMerchant(merchant);
        }else {
            boolean b = merchantService.saveMerchant(merchant);
            Merchant byTitle = merchantMapper.findByTitle(merchant);
            Callout callout = new Callout();
            callout.setContent(merchant.getTitle());
            callout.setMerchantId(byTitle.getId());
            calloutService.insertCallout(callout);
            return true;

        }
    }

    @DeleteMapping("/{id}")
    public boolean delete(@PathVariable Integer id){
        calloutService.deleteCallout(id);
        return merchantService.removeById(id);
    }

    @PostMapping("/del/batch")
    public boolean deleteBatch(@RequestBody List<Integer> ids){
        for (int i = 0; i < ids.size(); i++) {
            int id = (int) ids.get(i);
            String merchantId = String.valueOf(id);
            List<FilesDto> fileUUIDList = filesMapper.getFileUUIDList(merchantId);
            System.out.println(fileUUIDList);
            for (int j = 0 ; j < fileUUIDList.size() ; j ++){
                String iPath = fileUUIDList.get(j).getIconPath().substring(31);
                String url= fileUploadPath + iPath;
                File file = new File(url);
                boolean isOK = file.exists();
                if (isOK){
                    file.delete();
                }

            }
            filesMapper.removeByMerchantId(merchantId);
        }
        for (int m = 0; m < ids.size(); m++) {
            int id = ids.get(m);
            calloutService.deleteCallout(id);
        }
        return merchantService.removeByIds(ids);
    }
    //分页查询
    @GetMapping("/page")
    public IPage<Merchant> findPage(@RequestParam Integer pageNum,
                                @RequestParam Integer pageSize,
                                @RequestParam(defaultValue = "") String title,
                                @RequestParam(defaultValue = "") String email,
                                @RequestParam(defaultValue = "") String address){
        IPage<Merchant> page = new Page<>(pageNum , pageSize);
        QueryWrapper<Merchant> queryWrapper = new QueryWrapper<>();
        if (!"".equals(title)){
            queryWrapper.like("title",title);
        }

        return merchantService.page(page, queryWrapper);
    }
    //分页查询接口
    //@RequestParam接收 ?pageNum=1&pageSize=10
    //limit第一个参数(pageNum-1)*pageSize 第二个参数pageSize
//    @GetMapping("/page")
//    public Map<String,Object> findPage(@RequestParam Integer pageNum,
//                                       @RequestParam Integer pageSize,
//                                       @RequestParam String username){
//        pageNum = (pageNum-1)*pageSize;
//        Integer total = userMapper.selectTotal(username);
//        List<User> data = userMapper.selectPage(pageNum, pageSize,username);
//        Map<String,Object> res = new HashMap<>();
//        res.put("total",total);
//        res.put("data",data);
//        return res;
//    }


    @GetMapping("export")
    public void export(HttpServletResponse response) throws Exception{
        //从数据库查出所有数据
        List<Merchant> list = merchantService.list();
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("username","用户名");
        writer.addHeaderAlias("password","密码");
        writer.addHeaderAlias("nickname","昵称");
        writer.addHeaderAlias("email","邮箱");
        writer.addHeaderAlias("phone","电话");
        writer.addHeaderAlias("address","地址");
        writer.addHeaderAlias("createTime","创建时间");
        writer.addHeaderAlias("avatarUrl","头像");

        writer.write(list,true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        String fileName = URLEncoder.encode("商户信息","UTF-8");
        response.setHeader("Content-Disposition","attachment;filename=" + fileName + ".xlsx");

        ServletOutputStream out = response.getOutputStream();
        writer.flush(out,true);
        out.close();
        writer.close();

    }

//    @PostMapping("/import")
//    public boolean imp(MultipartFile file) throws Exception{
//        InputStream inputStream = file.getInputStream();
//        ExcelReader reader = ExcelUtil.getReader(inputStream);
////        List<User> list = reader.readAll(User.class);
//        List<List<Object>> list = reader.read(1);
//        List<Merchant> merchants = CollUtil.newArrayList();
//        for (List<Object> row : list){
//            User user = new User();
//            user.setUsername(row.get(0).toString());
//            user.setPassword(row.get(1).toString());
//            user.setNickname(row.get(2).toString());
//            user.setEmail(row.get(3).toString());
//            user.setPhone(row.get(4).toString());
//            user.setAddress(row.get(5).toString());
//            user.setAvatarUrl(row.get(6).toString());
//            users.add(user);
//
//        }
//        merchantService.saveBatch(users);
//        return true;
//    }

}
