package com.gjz.xiaoheizi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gjz.xiaoheizi.controller.dto.IntroduceFilesDto;
import com.gjz.xiaoheizi.controller.dto.NotificationFilesDto;
import com.gjz.xiaoheizi.controller.dto.ScenicSpotFilesDto;
import com.gjz.xiaoheizi.entity.*;
import com.gjz.xiaoheizi.mapper.NotificationFilesMapper;
import com.gjz.xiaoheizi.mapper.NotificationMapper;
import com.gjz.xiaoheizi.service.NotificationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.List;

/**
 * @author Guan
 * @date 2023/2/9
 * discription
 */
@RestController
@RequestMapping("/notification")
public class NotificationController {

    @Autowired
    private NotificationService notificationService;

    @Autowired
    private NotificationMapper notificationMapper;

    @Autowired
    private NotificationFilesMapper notificationFilesMapper;

    @Value("${files.upload.path}")
    private String fileUploadPath;

    @GetMapping("/page")
    public IPage<Notification> findPage(@RequestParam Integer pageNum,
                                        @RequestParam Integer pageSize,
                                        @RequestParam(defaultValue = "") String title) {
        IPage<Notification> page = new Page<>(pageNum, pageSize);
        QueryWrapper<Notification> queryWrapper = new QueryWrapper<>();
        if (!"".equals(title)) {
            queryWrapper.like("title", title);
        }

        return notificationService.page(page, queryWrapper);
    }

    //新增和修改
    @PostMapping("/save")
    public boolean save(@RequestBody Notification notification) {
        boolean b = notificationService.saveOrUpdate(notification);
        return b;
    }

    @PostMapping("/files")
    public boolean saveFiles(@RequestBody NotificationFilesDto notificationFilesDto){
        return notificationMapper.saveOne(notificationFilesDto);
    }

    @GetMapping("/status/{id}")
    public void statusChange(@PathVariable Integer id){
        Notification byId = notificationService.getById(id);
        int status = byId.getStatus();
        if (status == 0){
            status = 1;
        }else {
            status = 0;
        }
        byId.setStatus(status);
        notificationService.updateById(byId);
    }

    //删除
    @DeleteMapping("/{id}")
    public boolean delete(@PathVariable Integer id){
        return notificationService.removeById(id);
    }

    @DeleteMapping("/delicon/{notificationId}")
    public boolean deleteone(@PathVariable String notificationId){
        List<NotificationFilesDto> fileUUIDList = notificationFilesMapper.getFileUUIDList(notificationId);
        System.out.println(fileUUIDList);
        for (int i = 0 ; i < fileUUIDList.size() ; i ++){
            String iPath = fileUUIDList.get(i).getIconPath().substring(31);
            String url= fileUploadPath + iPath;
            File file = new File(url);
            boolean isOK = file.exists();
            if (isOK){
                file.delete();
            }

        }
        return notificationFilesMapper.removeByScenicspotId(notificationId);
    }

    //批量删除
    @PostMapping("/del/batch")
    public boolean deleteBatch(@RequestBody List<Integer> ids){
        for (int i = 0; i < ids.size(); i++){
            int id = ids.get(i);
            String notificationId = String.valueOf(id);
            List<NotificationFilesDto> fileUUIDList = notificationFilesMapper.getFileUUIDList(notificationId);
            for (int j = 0 ; j < fileUUIDList.size() ; j++){
                String iPath = fileUUIDList.get(j).getIconPath().substring(31);
                String url = fileUploadPath + iPath;
                File file = new File(url);
                boolean isOK = file.exists();
                if (isOK){
                    file.delete();
                }
            }
            notificationFilesMapper.removeByScenicspotId(notificationId);
        }
        return notificationService.removeByIds(ids);
    }


    //删除单张图片
    @Value("${files.upload.urlhead}")
    private String urlHead;
    @DeleteMapping("/icon/{fileUUID}")
    public boolean deleteIcon(@PathVariable String fileUUID){
        String url= fileUploadPath+ fileUUID;
        File file = new File(url);
        boolean isOK = file.exists();
        if (isOK){
            file.delete();
            String path = urlHead + fileUUID;
            notificationMapper.deleteByPath(path);
            System.out.println("122222222222222222222222222222");
            return true;
        }else{
            return false;
        }
    }

    //返回图片path
    @GetMapping("/files/{id}")
    public List<Files> findAllFilesPath(@PathVariable Integer id){
        return notificationMapper.findByIdFiles(id);
    }



}
