package com.gjz.xiaoheizi.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gjz.xiaoheizi.controller.dto.FilesDto;
import com.gjz.xiaoheizi.controller.dto.ScenicSpotFilesDto;
import com.gjz.xiaoheizi.entity.Files;
import com.gjz.xiaoheizi.entity.ScenicSpot;
import com.gjz.xiaoheizi.entity.ScenicSpotCallout;
import com.gjz.xiaoheizi.mapper.ScenicSpotFilesMapper;
import com.gjz.xiaoheizi.mapper.ScenicSpotMapper;
import com.gjz.xiaoheizi.service.ScenicSpotCalloutService;
import com.gjz.xiaoheizi.service.ScenicSpotService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.util.List;

/**
 * @author Guan
 * @date 2023/2/2
 * discription
 */
@RestController
@RequestMapping("/scenicspot")
public class ScenicSpotController {

    @Autowired
    private ScenicSpotService scenicSpotService;

    @Autowired
    private ScenicSpotCalloutService scenicSpotCalloutService;

    @Autowired
    private ScenicSpotFilesMapper scenicSpotFilesMapper;

    @Autowired
    private ScenicSpotMapper scenicSpotMapper;

    @Value("${files.upload.path}")
    private String fileUploadPath;

    //分页
    @GetMapping("/page")
    public IPage<ScenicSpot> findPage(@RequestParam Integer pageNum,
                                      @RequestParam Integer pageSize,
                                      @RequestParam(defaultValue = "") String title){
        IPage<ScenicSpot> page = new Page<>(pageNum , pageSize);
        QueryWrapper<ScenicSpot> queryWrapper = new QueryWrapper<>();
        if (!"".equals(title)){
            queryWrapper.like("title",title);
        }
        return scenicSpotService.page(page,queryWrapper);
    }

    //删除
    @DeleteMapping("/{id}")
    public boolean delete(@PathVariable Integer id){
        scenicSpotCalloutService.deleteSpotCallout(id);
        return scenicSpotService.removeById(id);
    }


    @DeleteMapping("/delicon/{scenicspotId}")
    public boolean deleteone(@PathVariable String scenicspotId){
        List<ScenicSpotFilesDto> fileUUIDList = scenicSpotFilesMapper.getFileUUIDList(scenicspotId);
        System.out.println(fileUUIDList);
        for (int i = 0 ; i < fileUUIDList.size() ; i ++){
            String iPath = fileUUIDList.get(i).getIconPath().substring(31);
            String url= fileUploadPath + iPath;
            File file = new File(url);
            boolean isOK = file.exists();
            if (isOK){
                file.delete();
            }

        }
        return scenicSpotFilesMapper.removeByScenicspotId(scenicspotId);
    }

    //批量删除
    @PostMapping("/del/batch")
    public boolean deleteBatch(@RequestBody List<Integer> ids){
        for (int i = 0; i < ids.size(); i++){
            int id = ids.get(i);
            String scenicspotId = String.valueOf(id);
            List<ScenicSpotFilesDto> fileUUIDList = scenicSpotFilesMapper.getFileUUIDList(scenicspotId);
            for (int j = 0 ; j < fileUUIDList.size() ; j++){
                String iPath = fileUUIDList.get(j).getIconPath().substring(31);
                String url = fileUploadPath + iPath;
                File file = new File(url);
                boolean isOK = file.exists();
                if (isOK){
                    file.delete();
                }
            }
            scenicSpotFilesMapper.removeByScenicspotId(scenicspotId);
        }
        for (int m = 0; m < ids.size(); m++){
            int id = ids.get(m);
            scenicSpotCalloutService.deleteSpotCallout(id);
        }
        return scenicSpotService.removeByIds(ids);
    }

    //修改标志位
    @GetMapping("/status/{id}")
    public void statusChange(@PathVariable Integer id){
        ScenicSpot byId = scenicSpotService.getById(id);
        int status = byId.getStatus();
        if (status == 0){
            status = 1;
        }else {
            status = 0;
        }
        byId.setStatus(status);
        scenicSpotService.updateById(byId);
    }

    //新增和修改
    @PostMapping
    public boolean save(@RequestBody ScenicSpot scenicSpot){
        if (scenicSpotService.getById(scenicSpot.getId()) != null){
            ScenicSpotCallout scenicSpotCallout = new ScenicSpotCallout();
            scenicSpotCallout.setContent(scenicSpot.getTitle());
            scenicSpotCallout.setScenicspotId(scenicSpot.getId());
            scenicSpotCalloutService.updateContent(scenicSpotCallout);
            return scenicSpotService.saveScenicSpot(scenicSpot);
        }else {
            boolean b = scenicSpotService.saveScenicSpot(scenicSpot);
            ScenicSpot byTitle = scenicSpotMapper.findByTitle(scenicSpot);
            ScenicSpotCallout scenicSpotCallout = new ScenicSpotCallout();
            scenicSpotCallout.setContent(scenicSpot.getTitle());
            scenicSpotCallout.setScenicspotId(byTitle.getId());
            scenicSpotCalloutService.insertCallout(scenicSpotCallout);
            return true;
        }
    }


    @PostMapping("/files")
    public boolean saveFiles(@RequestBody ScenicSpotFilesDto scenicSpotFilesDto){
        return scenicSpotMapper.saveOneFile(scenicSpotFilesDto);
    }

    //删除单张图片
    @Value("${files.upload.urlhead}")
    private String urlHead;

    @DeleteMapping("/icon/{fileUUID}")
    public boolean deleteIcon(@PathVariable String fileUUID){
        String url= fileUploadPath+ fileUUID;
        File file = new File(url);
        boolean isOK = file.exists();
        if (isOK){
            file.delete();
            String path = urlHead + fileUUID;
            scenicSpotMapper.deleteByPath(path);
            System.out.println("122222222222222222222222222222");
            return true;
        }else{
            return false;
        }
    }

    @GetMapping("/files/{id}")
    public List<Files> findAllFilesPath(@PathVariable Integer id){
        return scenicSpotMapper.findByIdFiles(id);
    }


}
