package com.gjz.xiaoheizi.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelPicUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gjz.xiaoheizi.common.JwtUtils;
import com.gjz.xiaoheizi.common.R;
import com.gjz.xiaoheizi.controller.dto.ChangeUserPassword;
import com.gjz.xiaoheizi.controller.dto.UserDto;
import com.gjz.xiaoheizi.entity.Merchant;
import com.gjz.xiaoheizi.entity.User;
import com.gjz.xiaoheizi.mapper.MerchantMapper;
import com.gjz.xiaoheizi.mapper.UserMapper;
import com.gjz.xiaoheizi.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author Guan
 * @date 2022/10/12
 * discription
 */
@RestController
@RequestMapping("/user")
public class UserController {
    @Autowired
    private UserService userService;
//
//    @Autowired
//    private MerchantMapper merchantMapper;
//
//    @GetMapping("/{ids}")
//    public List<Merchant> getsss(@PathVariable Integer ids) {
//        List<Merchant> byId = merchantMapper.findById(ids);
//        return byId;
//    }

    //查询所有
    @GetMapping
    public List<User> findAll(){
        return userService.list();
    }

    //login
    @PostMapping("/login")
    public R login(HttpServletRequest request, @RequestBody UserDto userDto){
        String username = userDto.getUsername();
        String password = userDto.getPassword();
        if (StrUtil.isBlank(username) || StrUtil.isBlank(password)){
            return R.error();
        }
        boolean login = userService.login(userDto);
        if (login != false){
//            request.getSession().setAttribute("employee", userDto.getUsername());
//            System.out.println("***********************************************");
//            System.out.println(request.getSession().getAttribute("employee"));
//            System.out.println(request.getSession().getId());
            QueryWrapper<User> wrapper = new QueryWrapper<>();
            wrapper.eq("username",userDto.getUsername());
            User one = userService.getOne(wrapper);

            String token = JwtUtils.getJwtToken(one.getId().toString(), one.getUsername());
            return R.ok().data("userInfo",one).data("token",token);
        }
        return R.error();
    }
    @PostMapping("/logout")
    public R logout(HttpServletRequest request){
        request.getSession().removeAttribute("employee");
        return R.ok().message("退出成功");
    }

    //修改密码
    @PostMapping("/changePassword")
    public R changePassword(@RequestBody ChangeUserPassword changeUserPassword){
        //判断原始密码是否正确
        String oldMd5 = DigestUtils.md5DigestAsHex(changeUserPassword.getOldPassword().getBytes());
        QueryWrapper<User> wrapper = new QueryWrapper<>();
        wrapper.eq("username",changeUserPassword.getUsername());
        User one = userService.getOne(wrapper);
        if (!oldMd5.equals(one.getPassword())){
            return R.error().message("密码错误");
        }
        //到此验证密码成功
        String newPassword = changeUserPassword.getNewPassword();
        String newMd5Password = DigestUtils.md5DigestAsHex(newPassword.getBytes());
        one.setPassword(newMd5Password);
        userService.saveOrUpdate(one);
        return R.ok();
    }



    //新增和修改
    @PostMapping
    public boolean save(@RequestBody User user){
        return userService.saveUser(user);
    }

    @DeleteMapping("/{id}")
    public boolean delete(@PathVariable Integer id){
        return userService.removeById(id);
    }

    @PostMapping("/del/batch")
    public boolean deleteBatch(@RequestBody List<Integer> ids){
        return userService.removeByIds(ids);
    }



    //分页查询接口
    //@RequestParam接收 ?pageNum=1&pageSize=10
    //limit第一个参数(pageNum-1)*pageSize 第二个参数pageSize
//    @GetMapping("/page")
//    public Map<String,Object> findPage(@RequestParam Integer pageNum,
//                                       @RequestParam Integer pageSize,
//                                       @RequestParam String username){
//        pageNum = (pageNum-1)*pageSize;
//        Integer total = userMapper.selectTotal(username);
//        List<User> data = userMapper.selectPage(pageNum, pageSize,username);
//        Map<String,Object> res = new HashMap<>();
//        res.put("total",total);
//        res.put("data",data);
//        return res;
//    }
    //分页查询
    @GetMapping("/page")
    public IPage<User> findPage(@RequestParam Integer pageNum,
                                @RequestParam Integer pageSize,
                                @RequestParam(defaultValue = "") String username,
                                @RequestParam(defaultValue = "") String email,
                                @RequestParam(defaultValue = "") String address){
        IPage<User> page = new Page<>(pageNum , pageSize);
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        if (!"".equals(username)){
            queryWrapper.like("username",username);
        }
        if (!"".equals(email)){
            queryWrapper.like("email",email);
        }
        if (!"".equals(address)){
            queryWrapper.like("address",address);
        }



        return userService.page(page, queryWrapper);
    }

    @GetMapping("export")
    public void export(HttpServletResponse response) throws Exception{
        //从数据库查出所有数据
        List<User> list = userService.list();
        ExcelWriter writer = ExcelUtil.getWriter(true);
        writer.addHeaderAlias("username","用户名");
        writer.addHeaderAlias("password","密码");
        writer.addHeaderAlias("nickname","昵称");
        writer.addHeaderAlias("email","邮箱");
        writer.addHeaderAlias("phone","电话");
        writer.addHeaderAlias("address","地址");
        writer.addHeaderAlias("createTime","创建时间");
        writer.addHeaderAlias("avatarUrl","头像");

        writer.write(list,true);

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8");
        String fileName = URLEncoder.encode("商户信息","UTF-8");
        response.setHeader("Content-Disposition","attachment;filename=" + fileName + ".xlsx");

        ServletOutputStream out = response.getOutputStream();
        writer.flush(out,true);
        out.close();
        writer.close();

    }

    @PostMapping("/import")
    public boolean imp(MultipartFile file) throws Exception{
        InputStream inputStream = file.getInputStream();
        ExcelReader reader = ExcelUtil.getReader(inputStream);
//        List<User> list = reader.readAll(User.class);
        List<List<Object>> list = reader.read(1);
        List<User> users = CollUtil.newArrayList();
        for (List<Object> row : list){
            User user = new User();
            user.setUsername(row.get(0).toString());
            user.setPassword(row.get(1).toString());
            user.setNickname(row.get(2).toString());
            user.setEmail(row.get(3).toString());
            user.setPhone(row.get(4).toString());
            user.setAddress(row.get(5).toString());
            user.setAvatarUrl(row.get(6).toString());
            users.add(user);

        }
        userService.saveBatch(users);
        return true;
    }


}
